function output = taGasStimFixedTimeEndpoint(input)

% Variables at Start and Terminus of Phase 1
t0 = input.phase(1).initialtime;
tf = input.phase(1).finaltime;
x0 = input.phase(1).initialstate;
xf = input.phase(1).finalstate;

integral = input.phase(1).integral;

input.phase(1).state = [x0;xf];
input.phase(1).time = [t0;tf];
[grfs,residuals] = getGrfs(input);

% Constraints
if input.auxdata.handLoadControls
    output.eventgroup(1).event = ([x0(1:62)-xf(1:62) diff(grfs) residuals]-...
        (input.auxdata.eventMax+input.auxdata.eventMin)/2)...
        ./(input.auxdata.eventMax-input.auxdata.eventMin);
else
    output.eventgroup.event = ([x0([7:31 38:62])-xf([7:31 38:62])  diff(grfs) diff(handForces) residuals]-...
        (input.auxdata.eventMax+input.auxdata.eventMin)/2)...
        ./(input.auxdata.eventMax-input.auxdata.eventMin);
end


% Objective
% propulsiveForceSqDiff = (integral(97) - integral(98))^2;
propulsiveForceSqDiff = (integral(32) - integral(33))^2;

% q = 1e0*(sum(integral([13,20,21:31]))*0 ... %13,20,21:23,25,29
%     +sum(integral(32:37).*[0,0.1,0.1,0,0.1,0.1])*0 ...
%     +sum(integral(38:43).*[0.2,0.2,0,0.2,0.2,0])*0 ...
%     +sum(integral(44:55))*0 ...
%     +sum(integral(56:65))*0 ...
%     +sum(integral(66:96))*0.001 ...
%     +propulsiveForceSqDiff*0.1);
q = 1e1*(sum(integral(1:31))*0.001 ...
    +propulsiveForceSqDiff*0.1);


output.objective = q;
end


%% Function Definition
function [grfs,residuals] = getGrfs(input)

% rescale everything back to original
nPts = length(input.phase.time);
onesCol = ones(nPts,1);

% input.phase.time = (input.phase.time)*(input.auxdata.tMax-input.auxdata.tMin)+(input.auxdata.tMax+input.auxdata.tMin)/2;
input.phase.time = input.phase.time*input.auxdata.timeScale;
input.phase.state = (input.phase.state).*(onesCol*(input.auxdata.stateMax-input.auxdata.stateMin))+onesCol*(input.auxdata.stateMax+input.auxdata.stateMin)/2;


%% Load parameters from the auxilary data
tScale = input.auxdata.tScale;

mu_s = 0; % Coefficient of static friction for foot-ground contact
mu_d = 1;%input.phase.parameter(1,9);%input.auxdata.mu;%parameters(1,1); % Coefficient of dynamic friction
mu_v = 0; % Coefficient of viscous friction
kVal = input.auxdata.kVal;%input.auxdata.Kval; % Spring stiffnesses for foot ground contact
cVal = input.auxdata.cVal; % Spring damping
yVal = input.auxdata.yVal;
springMat = input.auxdata.springMat;
springBodyMat = input.auxdata.springBodyMat;
numSpringsBody = input.auxdata.numSpringsBody;

% SpringMat(7:end,2) = SpringMat(7:end,2)+Yval(1);
springMat(7:6+input.auxdata.numSprings/2,2) = springMat(7:6+input.auxdata.numSprings/2,2)+yVal(1);
springMat(7+input.auxdata.numSprings/2:end,2) = springMat(7+input.auxdata.numSprings/2:end,2)+yVal(2);

% Transition velocity between static and dynamic friction
latchvel = input.auxdata.latchvel;


%% Initialize Variables

t = input.phase(1).time;

if ~exist('mkrData','var')
    mcr = fnval(input.auxdata.spMcr,t)';
    mcl = fnval(input.auxdata.spMcl,t)';
    handLoadsOptR = fnval(input.auxdata.spHandLoadsR,input.auxdata.time(end)*t/t(end))';
    handLoadsOptL = fnval(input.auxdata.spHandLoadsL,input.auxdata.time(end)*t/t(end))';
elseif size(mkrData,1) ~= nPts
    mcr = fnval(input.auxdata.spMcr,t)';
    mcl = fnval(input.auxdata.spMcl,t)';
    handLoadsOptR = fnval(input.auxdata.spHandLoadsR,input.auxdata.time(end)*t/t(end))';
    handLoadsOptL = fnval(input.auxdata.spHandLoadsL,input.auxdata.time(end)*t/t(end))';
end

%% Kinematics

% Set states
x = input.phase(1).state(:,1:31);
xp = input.phase(1).state(:,32:62);

% Calculate spring positions and velocities
[springPos, springVels] = opensimPointKin_v3(t,[x xp],springMat',springBodyMat);

% Store location of heel and toes body for tranferring GRFs
BodyPositions = springPos(:,1:12);
% mcr = SpringPos(:,13:15);
% mcl = SpringPos(:,16:18);
springPos(:,1:18) = [];
springVels(:,1:18) = [];

mcr(:,2) = 0;
mcl(:,2) = 0;

%% Calculate Ground Reaction Forces

% Calculate spring forces and moments about electrical centers
[footGRFsRMC, footGRFsLMC] = calcGroundReactions_v3(springPos, springVels, ...
    numSpringsBody, mu_s, mu_d, mu_v, kVal, cVal, mcr, mcl, latchvel, tScale,input.auxdata.beltSpeed);

% Transfer moments to heel and toe origin for input into full body model
% This is necessary because the contact model ground reactions are
% calculated about some point in the room (force plate electrical centers),  
% and the model applies the loads at the origin of the foot and toes body
footGRFsR = footGRFsRMC;
footGRFsL = footGRFsLMC;

% Transfer to heel
footGRFsR(:,7:9) = cross(mcr-BodyPositions(:,1:3),footGRFsRMC(:,1:3),2)+footGRFsRMC(:,7:9);
footGRFsL(:,7:9) = cross(mcl-BodyPositions(:,7:9),footGRFsLMC(:,1:3),2)+footGRFsLMC(:,7:9);

% Transfer to toe
footGRFsR(:,10:12) = cross(mcr-BodyPositions(:,4:6),footGRFsRMC(:,4:6),2)+footGRFsRMC(:,10:12);
footGRFsL(:,10:12) = cross(mcl-BodyPositions(:,10:12),footGRFsLMC(:,4:6),2)+footGRFsLMC(:,10:12);


%% Dynamics

% Extract for input into full body dynamic model
torques = zeros(nPts,25);%Set to zero because they will be determined by inverse dynamics

% Formulate inputs into opensim (states and controls w/ GRFs)
appliedLoads = [torques footGRFsR footGRFsL handLoadsOptR handLoadsOptL];

% Set accelerations
xpp = input.phase.state(:,63:end);

% Calculate state derivatives and marker locations
[residualLoads] = opensimID_OMP(t,[x xp], xpp, appliedLoads,[1 0 0]);
residuals = residualLoads(end,1:6);


%% Transfer GRFs

% Calculate net GRFs at the electrical center for use in objective function
footGroundReactionsR = footGRFsR(:,[1:3 7:9])+footGRFsR(:,[4:6 10:12]);
footGroundReactionsL = footGRFsL(:,[1:3 7:9])+footGRFsL(:,[4:6 10:12]);

grfs = [footGroundReactionsR(:,1:3) footGroundReactionsL(:,1:3)];

end